@@ -19,6 +19,7 @@
 (require 'url-vars)
 
 (require 'cl-lib)
+(require 'ffap)
 (require 'files)
 (require 'ls-lisp)
 (require 'pp)
@@ -152,6 +153,12 @@ will return `lint/checkdoc' with a dash between two subcommands."
                                     (list script-file))
                  "/"))))
 
+(defun eask-command-check (version)
+  "Report error if the current command requires minimum VERSION."
+  (when (version< emacs-version version)
+    (eask-error "The command `%s' requires Emacs %s and above!"
+                (eask-command) version)))
+
 (defun eask-command-p (commands)
   "Return t if COMMANDS is the current command."
   (member (eask-command) (eask-listify commands)))
@@ -474,7 +481,8 @@ Arguments FNC and ARGS are used for advice `:around'."
            (url (format "https://raw.githubusercontent.com/emacs-eask/archives/master/%s/" name))
            (url-file (concat url file))
            (download-p)
-           (local-archive-p (string= name "local"))  ; exclude local elpa
+           ;; exclude local elpa
+           (local-archive-p (string= name eask--local-archive-name))
            (fmt (eask--action-format (length package-archives))))
       (unless (file-exists-p local-file)
         (eask-with-verbosity-override 'log
@@ -541,10 +549,25 @@ For arguments FUNC and DEPS, see function `mapc' for more information."
          (len (length deps))
          (fmt (eask--action-format len))
          (count 0))
-    (dolist (pkg deps)
+    (dolist (dep deps)
       (cl-incf count)
       (setq eask--action-prefix (format fmt count))
-      (funcall func pkg))))
+      (funcall func dep))))
+
+(defun eask--install-dep (dep)
+  "Custom install DEP."
+  (let ((name (car dep)))
+    (cond
+     ;; Install through the function `package-install-file'.
+     ((memq :file dep)
+      (let ((file (nth 2 dep)))
+        (eask-package-install-file name file)))
+     ;; Install through the function `package-vc-install'.
+     ((memq :vc dep)
+      (let ((spec (cdr (memq :vc dep))))
+        (eask-package-vc-install name spec)))
+     ;; Fallback to archive install.
+     (t (eask-package-install name)))))
 
 (defun eask--install-deps (dependencies msg)
   "Install DEPENDENCIES.
@@ -556,10 +579,11 @@ scope of the dependencies (it's either `production' or `development')."
          (len (length dependencies))
          (ies (eask--sinr len "y" "ies"))
          (pkg-installed (cl-remove-if #'package-installed-p names))
-         (installed (length pkg-installed)) (skipped (- len installed)))
+         (installed (length pkg-installed))
+         (skipped (- len installed)))
     (eask-log "Installing %s %s dependenc%s..." len msg ies)
     (eask-msg "")
-    (eask--package-mapc #'eask-package-install names)
+    (eask--package-mapc #'eask--install-dep dependencies)
     (eask-msg "")
     (eask-info "(Total of %s dependenc%s installed, %s skipped)"
                installed ies skipped)))
@@ -628,10 +652,12 @@ Argument PKG is the name of the package."
 (defmacro eask--pkg-process (pkg &rest body)
   "Execute BODY with PKG's related variables."
   (declare (indent 1) (debug t))
-  `(let* ((pkg-info (eask--pkg-transaction-vars ,pkg))
-          (pkg      (nth 0 pkg-info))
-          (name     (nth 1 pkg-info))
-          (version  (nth 2 pkg-info)))
+  `(let* ((pkg-info           (eask--pkg-transaction-vars ,pkg))
+          (pkg                (nth 0 pkg-info))
+          (name               (nth 1 pkg-info))
+          (version            (nth 2 pkg-info))
+          (installed-p        (package-installed-p pkg))
+          (should-reinstall-p (and installed-p (eask-force-p))))
      ,@body))
 
 (defmacro eask-with-archives (archives &rest body)
@@ -668,12 +694,54 @@ Argument BODY are forms for execution."
   "Return non-nil if package (PKG) is installable."
   (assq (eask-intern pkg) package-archive-contents))
 
+(defun eask-package-vc-install (pkg spec)
+  "To vc install the package (PKG) by argument SPEC."
+  (eask-defvc< 27 (eask-pkg-init))  ; XXX: remove this after we drop 26.x
+  (eask--pkg-process pkg
+    (cond
+     ((and installed-p (not should-reinstall-p))
+      (eask-msg "  - %sSkipping %s (%s)... already installed ✗"
+                eask--action-prefix
+                name version))
+     (t
+      (eask-with-progress
+        (format "  - %s%snstalling %s (%s)... " eask--action-prefix
+                (if should-reinstall-p "Rei" "I")
+                name version)
+        (eask-with-verbosity 'debug
+          ;; Handle `--force` flag.
+          (when should-reinstall-p (package-delete (eask-package-desc pkg t) t))
+          ;; Install it.
+          (apply #'package-vc-install spec))
+        "done ✓")))))
+
+(defun eask-package-install-file (pkg file)
+  "To FILE install the package (PKG)."
+  (eask-defvc< 27 (eask-pkg-init))  ; XXX: remove this after we drop 26.x
+  (eask--pkg-process pkg
+    (cond
+     ((and installed-p (not should-reinstall-p))
+      (eask-msg "  - %sSkipping %s (%s)... already installed ✗"
+                eask--action-prefix
+                name version))
+     (t
+      (eask-with-progress
+        (format "  - %s%snstalling %s (%s)... " eask--action-prefix
+                (if should-reinstall-p "Rei" "I")
+                name version)
+        (eask-with-verbosity 'debug
+          ;; Handle `--force` flag.
+          (when should-reinstall-p (package-delete (eask-package-desc pkg t) t))
+          ;; Install it.
+          (package-install-file (expand-file-name file)))
+        "done ✓")))))
+
 (defun eask-package-install (pkg)
   "Install the package (PKG)."
   (eask-defvc< 27 (eask-pkg-init))  ; XXX: remove this after we drop 26.x
   (eask--pkg-process pkg
     (cond
-     ((package-installed-p pkg)
+     ((and installed-p (not should-reinstall-p))
       (eask-msg "  - %sSkipping %s (%s)... already installed ✗"
                 eask--action-prefix
                 name version))
@@ -681,37 +749,31 @@ Argument BODY are forms for execution."
         (eask-pkg-init)
         (unless (eask-package-installable-p pkg)
           (eask-error "Package not installable `%s'; make sure the package archive (source) is included" pkg))))
-     ((when-let* ((desc (eask-package-desc pkg))
-                  (req-emacs (assoc 'emacs (package-desc-reqs desc)))
-                  (req-emacs (package-version-join (nth 0 (cdr req-emacs))))
-                  ((version< emacs-version req-emacs)))
-        (if (eask-strict-p)
-            (eask-error "  - %sSkipping %s (%s)... it requires Emacs %s and above ✗"
-                        eask--action-prefix
-                        pkg (eask-package--version-string pkg) req-emacs)
-          (eask-msg "  - %sSkipping %s (%s)... it requires Emacs %s and above ✗"
-                    eask--action-prefix
-                    name version (ansi-yellow req-emacs)))))
      (t
-      (eask--pkg-process pkg
-        (eask-with-progress
-          (format "  - %sInstalling %s (%s)... " eask--action-prefix name version)
-          (eask-with-verbosity 'debug
-            ;; XXX: Without ignore-errors guard, it will trigger error
-            ;;
-            ;;   Can't find library xxxxxxx.el
-            ;;
-            ;; But we can remove this after Emacs 28, since function `find-library-name'
-            ;; has replaced the function `signal' instead of the `error'.
-            (eask-ignore-errors (package-install pkg)))
-          "done ✓"))))))
+      (eask-with-progress
+        (format "  - %s%snstalling %s (%s)... " eask--action-prefix
+                (if should-reinstall-p "Rei" "I")
+                name version)
+        (eask-with-verbosity 'debug
+          ;; Handle `--force` flag.
+          (when should-reinstall-p (package-delete (eask-package-desc pkg t) t))
+          ;; XXX: Without ignore-errors guard, it will trigger error
+          ;;
+          ;;   Can't find library xxxxxxx.el
+          ;;
+          ;; But we can remove this after Emacs 28, since function `find-library-name'
+          ;; has replaced the function `signal' instead of the `error'.
+          ;;
+          ;; Install it.
+          (eask-ignore-errors (package-install pkg)))
+        "done ✓")))))
 
 (defun eask-package-delete (pkg)
   "Delete the package (PKG)."
   (eask-defvc< 27 (eask-pkg-init))  ; XXX: remove this after we drop 26.x
   (eask--pkg-process pkg
     (cond
-     ((not (package-installed-p pkg))
+     ((not installed-p)
       (eask-msg "  - %sSkipping %s (%s)... not installed ✗" eask--action-prefix name version))
      (t
       (eask--pkg-process pkg
@@ -726,17 +788,17 @@ Argument BODY are forms for execution."
   (eask-defvc< 27 (eask-pkg-init))  ; XXX: remove this after we drop 26.x
   (eask--pkg-process pkg
     (cond
-     ((not (package-installed-p pkg))
+     ;; You cannot reinstall the package that are not installed.
+     ((not installed-p)
       (eask-msg "  - %sSkipping %s (%s)... not installed ✗" eask--action-prefix name version))
      (t
       (eask-pkg-init)
-      (eask--pkg-process pkg
-        (eask-with-progress
-          (format "  - %sReinstalling %s (%s)... " eask--action-prefix name version)
-          (eask-with-verbosity 'debug
-            (package-delete (eask-package-desc pkg t) t)
-            (eask-ignore-errors (package-install pkg)))
-          "done ✓"))))))
+      (eask-with-progress
+        (format "  - %sReinstalling %s (%s)... " eask--action-prefix name version)
+        (eask-with-verbosity 'debug
+          (package-delete (eask-package-desc pkg t) t)
+          (eask-ignore-errors (package-install pkg)))
+        "done ✓")))))
 
 (defun eask-package-desc (name &optional current)
   "Build package description by its NAME.
@@ -1127,6 +1189,7 @@ This uses function `locate-dominating-file' to look up directory tree."
   "Try load the Eask-file in START-PATH."
   (when-let* ((files (eask--find-files start-path))
               (file (car files)))
+    ;; Revert printing behaviour when loading Eask-file.
     (eask--unsilent (eask-file-load file))))
 
 (defmacro eask--with-hooks (&rest body)
@@ -1160,12 +1223,14 @@ This uses function `locate-dominating-file' to look up directory tree."
   (let ((inhibit-config (eask-quick-p)))
     (eask-with-progress
       (ansi-green "Loading configuration... ")
-      (eask-with-verbosity 'all
+      ;; Revert printing behaviour when loading user files.
+      (eask--unsilent
         (unless inhibit-config
+          ;; `early-init.el' is supported after 27.1
           (when (version<= "27" emacs-version)
-            (load early-init-file t))
-          (load eask-dot-emacs-file t)
-          (load user-init-file t)))
+            (load early-init-file t t))
+          (load eask-dot-emacs-file t t)
+          (load user-init-file t t)))
       (ansi-green (if inhibit-config "skipped ✗" "done ✓")))))
 
 (defmacro eask-start (&rest body)
@@ -1450,35 +1515,61 @@ argument COMMAND."
 (defun eask-f-source (name &optional location)
   "Add archive NAME with LOCATION."
   (when (symbolp name) (setq name (eask-2str name)))  ; ensure to string, accept symbol
+  ;; Handle local archive.
+  (when (equal name eask--local-archive-name)
+    (eask-error "Invalid archive name `%s'" name))
+  ;; Handle multiple same archive name!
   (when (assoc name package-archives)
     (eask-error "Multiple definition of source `%s'" name))
   (setq location (eask-source-url name location))
   (unless location (eask-error "Unknown package archive `%s'" name))
   (add-to-list 'package-archives (cons name location) t))
 
-(defun eask-f-source-priority (archive-id &optional priority)
-  "Add PRIORITY for to ARCHIVE-ID."
-  (add-to-list 'package-archive-priorities (cons archive-id priority) t))
+(defun eask-f-source-priority (name &optional priority)
+  "Add PRIORITY for to NAME."
+  (when (symbolp name) (setq name (eask-2str name)))  ; ensure to string, accept symbol
+  (add-to-list 'package-archive-priorities (cons name priority) t))
 
 (defvar eask-depends-on-recipe-p nil
   "Set to t if package depends on recipe.")
 
+(defvar eask--local-archive-name "local"
+  "The local archive name.")
+
 (defun eask--setup-dependencies ()
   "Setup dependencies list."
   (setq eask-depends-on (reverse eask-depends-on)
         eask-depends-on-dev (reverse eask-depends-on-dev))
   (when eask-depends-on-recipe-p
     (eask-with-progress
-      "✓ Checking local archives... "
+      (format "✓ Checking local archives %s... "
+              (ansi-magenta eask--local-archive-name))
       (eask-with-verbosity 'debug
-        (add-to-list 'package-archives `("local" . ,github-elpa-archive-dir) t)
-        ;; If the local archives is added, we set the priority to a very
-        ;; high number so user we always use the specified dependencies!
-        (add-to-list 'package-archive-priorities `("local" . 90) t))
+        ;; Make sure can be customized by `source'
+        (unless (assoc eask--local-archive-name package-archives)
+          (add-to-list 'package-archives
+                       `(,eask--local-archive-name . ,github-elpa-archive-dir) t))
+        ;; Make sure can be customized by `source-priority'
+        (unless (assoc eask--local-archive-name package-archive-priorities)
+          ;; If the local archives is added, we set the priority to a very
+          ;; high number so user we always use the specified dependencies!
+          (add-to-list 'package-archive-priorities
+                       `(,eask--local-archive-name . 90) t)))
       "done!")))
 
 (add-hook 'eask-file-loaded-hook #'eask--setup-dependencies)
 
+(defun eask--check-depends-on (recipe)
+  "Return non-nil if RECIPE is invalid."
+  (let ((pkg (car recipe))
+        (minimum-version (cdr recipe)))
+    (cond ((member recipe eask-depends-on)
+           (eask-error "Define dependencies with the same name `%s'" pkg))
+          ((cl-some (lambda (rcp)
+                      (string= (car rcp) pkg))
+                    eask-depends-on)
+           (eask-error "Define dependencies with the same name `%s' with different version" pkg)))))
+
 (defun eask-f-depends-on (pkg &rest args)
   "Specify a dependency (PKG) of this package.
 
@@ -1497,17 +1588,27 @@ ELPA)."
         recipe)))
    ;; No argument specify
    ((<= (length args) 1)
-    (let* ((minimum-version (or (car args) "0"))
+    (let* ((minimum-version (car args))
            (recipe (list pkg minimum-version)))
-      (if (member recipe eask-depends-on)
-          (eask-error "Define dependencies with the same name `%s'" pkg)
+      (unless (eask--check-depends-on recipe)
+        (push recipe eask-depends-on))
+      recipe))
+   ;; File packages
+   ((memq :file args)
+    (let* ((recipe (append (list (intern pkg)) args)))
+      (unless (eask--check-depends-on recipe)
+        (push recipe eask-depends-on))
+      recipe))
+   ;; VC packages
+   ((memq :vc args)
+    (let* ((recipe (append (list (intern pkg)) args)))
+      (unless (eask--check-depends-on recipe)
         (push recipe eask-depends-on))
       recipe))
    ;; recipe are entered
    (t
     (let ((recipe (append (list (intern pkg)) args)))
-      (if (member recipe eask-depends-on)
-          (eask-error "Define dependencies with the same name `%s'" pkg)
+      (unless (eask--check-depends-on recipe)
         (push recipe eask-depends-on)
         (eask-load "extern/github-elpa")
         (eask-with-verbosity 'debug
@@ -1713,6 +1814,25 @@ For arguments MSG and ARGS, please see function `eask-msg' for the detials."
 Argument ARGS are direct arguments for functions `eask-error' or `eask-warn'."
   (apply (if (eask-strict-p) #'eask-error #'eask-warn) args))
 
+;;
+;;; Exit Code
+
+(defconst eask--exit-code
+  `((success . 0)   ; Unused
+    (failure . 1)   ; Catchall for general errors
+    (misuse  . 2))
+  "Exit code specification.")
+
+(defun eask-exit-code (key)
+  "Return the exit code by KEY symbol."
+  (alist-get key eask--exit-code))
+
+(defun eask--exit (&optional exit-code &rest _)
+  "Kill Emacs with EXIT-CODE (default 1)."
+  (kill-emacs (or (cond ((numberp exit-code) exit-code)
+                        ((symbolp exit-code) (eask-exit-code exit-code)))
+                  (eask-exit-code 'failure))))
+
 ;;
 ;;; Error Handling
 
@@ -1737,8 +1857,6 @@ Argument ARGS are direct arguments for functions `eask-error' or `eask-warn'."
   (declare (indent 0) (debug t))
   `(eask-ignore-errors (eask--silent-error ,@body)))
 
-(defun eask--exit (&rest _) "Send exit code." (kill-emacs 1))
-
 (defun eask--trigger-error ()
   "Trigger error event."
   (when (and (not eask--ignore-error-p)
@@ -1868,10 +1986,19 @@ Arguments FNC and ARGS are used for advice `:around'."
     (eask-msg (ansi-white (buffer-string)))
     (eask-msg (concat "''" (spaces-string max-column) "''"))))
 
-(defun eask-help (command)
-  "Show COMMAND's help instruction."
+(defun eask-help (command &optional print-or-exit-code)
+  "Show COMMAND's help instruction.
+
+When the optional variable PRINT-OR-EXIT-CODE is a number, it will exit with
+that code.  Set to non-nil would just print the help message without sending
+the exit code.  The default value `nil' will be replaced by `1'; therefore
+would send exit code of `1'."
   (let* ((command (eask-2str command))  ; convert to string
-         (help-file (concat eask-lisp-root "help/" command)))
+         (help-file (concat eask-lisp-root "help/" command))
+         ;; The default exit code is `2' since `eask-help' prints the help
+         ;; message on user error 99% of the time.
+         (print-or-exit-code (or print-or-exit-code
+                                 (eask-exit-code 'misuse))))
     (if (file-exists-p help-file)
         (with-temp-buffer
           (insert-file-contents help-file)
@@ -1879,15 +2006,19 @@ Arguments FNC and ARGS are used for advice `:around'."
             (let ((buf-str (eask--msg-displayable-kwds (buffer-string))))
               (erase-buffer)
               (insert buf-str))
-            (eask--help-display)))
-      (eask-error "Help manual missig %s" help-file))))
+            (eask--help-display))
+          ;; Exit with code if needed
+          (cond ((numberp print-or-exit-code)
+                 (eask--exit print-or-exit-code))
+                (t )))  ; Don't exit with anything else.
+      (eask-error "Help manual missing %s" help-file))))
 
 ;;
 ;;; Checker
 
 (defun eask--checker-existence ()
   "Return errors if required metadata is missing."
-  (unless eask-package (eask-error "Missing metadata package; make sure you have create Eask-file with $ eask init!")))
+  (unless eask-package (eask-error "Missing metadata package; make sure you have created an Eask-file with $ eask init!")))
 
 (defun eask--check-strings (fmt f p &rest args)
   "Test strings (F and P); then print FMT and ARGS if not equal."
